	INCLUDE equates.h

	IMPORT SerialIn
	IMPORT RumbleInterrupt
	IMPORT StartRumbleComs

	EXPORT waitframe
	EXPORT LZ77UnCompVram
	EXPORT CheckGBAVersion
	EXPORT memset_

	EXPORT IO_init
	EXPORT AGBinput

	EXPORT vbldummy
	EXPORT vblankfptr
	EXPORT vblankinterrupt

	EXPORT dmaoambuffer
	EXPORT PALETTEBUFF

;----------------------------------------------------------------------------
 AREA rom_code, CODE, READONLY ;-- - - - - - - - - - - - - - - - - - - - - -
;----------------------------------------------------------------------------

LZ77UnCompVram
	swi 0x120000
	bx lr

waitframe
	mov r0,#0				;don't wait if not necessary
	mov r1,#1				;VBL wait
	swi 0x040000			; Turn of CPU until VBLIRQ if not too late allready.
	bx lr

CheckGBAVersion
	ldr r0,=0x5AB07A6E		;Fool proofing
	mov r12,#0
	swi 0x0D0000			;GetBIOSChecksum
	ldr r1,=0xABBE687E		;Proto GBA, not really ;)
	cmp r0,r1
	moveq r12,#1
	ldr r1,=0xBAAE187F		;Normal GBA
	cmp r0,r1
	moveq r12,#2
	ldr r1,=0xBAAE1880		;Nintendo DS
	cmp r0,r1
	moveq r12,#4
	mov r0,r12
	bx lr


;----------------------------------------------------------------------------
IO_init	;(called from main.c) only need to call once
;----------------------------------------------------------------------------
	mov r1,#REG_BASE
	mov r0,#0x0008
	strh r0,[r1,#REG_DISPSTAT]	;vblank en


	add r2,r1,#REG_IE
	mov r0,#-1
	strh r0,[r2,#2]		;stop pending interrupts
	ldr r0,=irqhandler
	str r0,[r1,#-4]		;=AGB_IRQVECT
	ldr r0,=0x1081
	strh r0,[r2]		;key,vblank,serial enable.
	mov r0,#1
	strh r0,[r2,#8]		;master irq enable

	bx lr

;----------------------------------------------------------------------------
countseconds		;call StartRumbleComs() every second.
;----------------------------------------------------------------------------
	ldrb r0,fpschk
	subs r0,r0,#1
	movmi r0,#59
	strb r0,fpschk
	bxpl lr					;End if not 60 frames has passed

	str lr,[sp,#-4]!
	ldr r1,=StartRumbleComs
	adr lr,ret_
	bx r1
ret_

	ldr pc,[sp],#4
	bx lr

;----------------------------------------------------------------------------
fpschk	DCB 0
		DCB 0,0,0
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
	AREA wram_code1, CODE, READWRITE
irqhandler	;r0-r3,r12 are safe to use
;----------------------------------------------------------------------------
	mov r2,#REG_BASE
	mov r3,#REG_BASE
	ldr r1,[r2,#REG_IE]!
	and r1,r1,r1,lsr#16	;r1=IE&IF
	ldrh r0,[r3,#-8]
	orr r0,r0,r1
	strh r0,[r3,#-8]

		;---these CAN'T be interrupted
		ands r0,r1,#0x80
		strneh r0,[r2,#2]		;IF clear
		ldrne r12,serialfptr
		bxne r12
		;---

		;---these CAN be interrupted
		ands r0,r1,#0x01
		ldrne r12,vblankfptr
		bne jmpintr
		;----
		adreq r12,irq0
		moveq r0,r1		;if unknown interrupt occured clear it.
jmpintr
	strh r0,[r2,#2]		;IF clear

	mrs r3,spsr
	stmfd sp!,{r3,lr}
	mrs r3,cpsr
	bic r3,r3,#0x9f
	orr r3,r3,#0x1f			;--> Enable IRQ . Set CPU mode to System.
	msr cpsr_cf,r3
	stmfd sp!,{lr}
	adr lr,irq0

	mov pc,r12


irq0
	ldmfd sp!,{lr}
	mrs r3,cpsr
	bic r3,r3,#0x9f
	orr r3,r3,#0x92        		;--> Disable IRQ. Set CPU mode to IRQ
	msr cpsr_cf,r3
	ldmfd sp!,{r0,lr}
	msr spsr_cf,r0
vbldummy
	bx lr
;----------------------------------------------------------------------------
vblankfptr DCD vbldummy			;later switched to vblankinterrupt
serialfptr DCD RumbleInterrupt

vblankinterrupt;
;----------------------------------------------------------------------------
	stmfd sp!,{r4-r7,globalptr,lr}

	bl countseconds


	mov r1,#REG_BASE
	strh r1,[r1,#REG_DM3CNT_H]		;DMA3 stop

	add r2,r1,#REG_DM3SAD

	ldr r4,dmaoambuffer				;DMA3 src, OAM transfer:
	mov r5,#AGB_OAM					;DMA3 dst
	mov r6,#0x84000000				;noIRQ 32bit incsrc incdst
	orr r6,r6,#0x100				;256 words (1024 bytes)
	stmia r2,{r4-r6}				;DMA3 go

	ldr r4,=PALETTEBUFF				;DMA3 src, Palette transfer:
	mov r5,#AGB_PALETTE				;DMA3 dst
	mov r6,#0x84000000				;noIRQ 32bit incsrc incdst
	orr r6,r6,#0x100				;256 words (1024 bytes)
	stmia r2,{r4-r6}				;DMA3 go


	mov r0,#0x1140					;1d sprites, BG0 enable. OBJ enable.
	strh r0,[r1,#REG_DISPCNT]		;set value manually

exit_vbl
	ldmfd sp!,{r4-r7,globalptr,pc}


;----------------------------------------------------------------------------
memset_ ;r0=dest r1=data r2=word count
;	exit with r0 & r1 unchanged, r2=0
;----------------------------------------------------------------------------
	subs r2,r2,#1
	strpl r1,[r0,r2,lsl#2]
	bhi memset_
	bx lr
;----------------------------------------------------------------------------

dmaoambuffer DCD OAM_BUFFER1

;----------------------------------------------------------------------------
	AREA wram_globals0, CODE, READWRITE

AGBinput			;this label here for main.c to use
	DCD 0 			;AGBjoypad (why is this in gfx.s again?  um.. i forget)


;----------------------------------------------------------------------------
	END
