#include <stdio.h>
#include <string.h>
#include "gba.h"

extern u32 font;				//from boot.s
extern u32 fontpal;				//from boot.s
extern u32 *vblankfptr;			//from io.s
extern u32 vbldummy;			//from io.s
extern u32 vblankinterrupt;		//from io.s


//asm calls
void IO_init(void);				//io.s
void LZ77UnCompVram(u32 *source,u16 *destination);		//io.s
void waitframe(void);			//io.s
int CheckGBAVersion(void);		//io.s

void cls(int);
void initpalette(void);
void splash(void);
void drawtext(int,char*,int);
void setdarknessgs(int dark);
void setbrightnessall(int light);
void sleepy(int);
void StartPong(void);				//pong.c

const unsigned __fp_status_arm=0x40070000;
u8 *textstart;//points to first PCE rom (initialized by boot.s)

char gameboyplayer=0;
char gbaversion;


void C_entry() {
	int i;
	gbaversion=CheckGBAVersion();
	vblankfptr=&vbldummy;
	IO_init();


	splash();


	if(REG_DISPCNT==FORCE_BLANK)	//is screen OFF?
		REG_DISPCNT=0;				//screen ON
	*MEM_PALETTE=0x7FFF;			//white background
	REG_BLDCNT=0x00ff;				//brightness decrease all
	for(i=0;i<17;i++) {
		REG_BLDY=i;					//fade to black
		waitframe();
	}
	*MEM_PALETTE=0;					//black background (avoids blue flash when doing multiboot)
	REG_DISPCNT=0;					//screen ON, MODE0
	vblankfptr=&vblankinterrupt;

	//load font
	LZ77UnCompVram(&font,(u16*)0x6002400);
	initpalette();
	cls(1);
	REG_BG0HOFS=0x0;
	REG_BG0CNT=0x4600;				//16color 512x256 CHRbase0 SCRbase6 Priority0
	setdarknessgs(0);				//Lighten screen
	while(1){
		drawtext(10,"         Press Start",0);
		waitframe();				//(polling REG_P1 too fast seems to cause problems)
		i=~REG_P1;
		if(i&START){
			cls(1);
			StartPong();
			cls(1);
		}
	}

}


//show splash screen
void splash() {
	int i;

	REG_DISPCNT=FORCE_BLANK;	//screen OFF
	memcpy((u16*)MEM_VRAM,(u16*)textstart,240*160*2);
	waitframe();
	REG_BG2CNT=0x0000;
	REG_DISPCNT=BG2_EN|MODE3;
	for(i=16;i>=0;i--) {	//fade from white
		setbrightnessall(i);
		waitframe();
	}
	for(i=0;i<150;i++) {	//wait 2.5 seconds
		waitframe();
		if (REG_P1==0x030f){
			gameboyplayer=1;
			gbaversion=3;
		}
	}
}



void initpalette(void) {
	//load palette
	memcpy((void*)(PAL_BUFF+0x80),&fontpal,64);
}


void cls(int chrmap) {
	int i=0,len=0x200;
	u32 *scr=(u32*)SCREENBASE;
	if(chrmap>=2)
		len=0x400;
	if(chrmap==2)
		i=0x200;
	for(;i<len;i++)				//512x256
		scr[i]=0x01200120;
	REG_BG2VOFS=0;
}

void drawtext(int row,char *str,int hilite) {
	u16 *here=SCREENBASE+row*32;
	int i=0;

	*here=hilite?0x412a:0x4120;
	hilite=(hilite<<12)+0x4100;
	here++;
	while(str[i]>=' ') {
		here[i]=str[i]|hilite;
		i++;
	}
	for(;i<31;i++)
		here[i]=0x0120;
}

void setdarknessgs(int dark) {
	REG_BLDCNT=0x01f3;				//darken game screen
	REG_BLDY=dark;					//Darken screen
	REG_BLDALPHA=(0x10-dark)<<8;	//set blending for OBJ affected BG0
}

void setbrightnessall(int light) {
	REG_BLDCNT=0x00bf;				//brightness increase all
	REG_BLDY=light;
}

void sleepy(int sec) {
	int i;

	for(i=0;i<sec*60;i++)
		waitframe();
}

