#include <stdio.h>
#include <string.h>
#include "gba.h"

extern u32 AGBinput;			//from vdc.s
extern u16 *dmaoambuffer;
extern u16 PALETTEBUFF[];

void drawtext(int,char*,int);	//main.c
void cls(int);					//from main.c
void sleepy(int);				//from main.c

void waitframe(void);			//io.s
void memset_(void *dest,int data,int len);

extern u32 DoRumble;			//from rumble.c

void PlayStartSound(void);
void PlayPong(void);
void CountScore(void);
void GameOver(void);
void UpdateSprites(int playerpos, int cpupos, int ballxpos, int ballypos);
int abs(int i);

static int state;
static int cpu_player;		// score for? cpu = 1, player = 2.
static int cpuscore;		//
static int playerscore;		//

void StartPong(){
	state = 0;

	while(1){
		switch (state){
			case 0:
				PlayStartSound();
				break;
			case 1:
				PlayPong();
				break;
			case 2:
				CountScore();
				break;
			case 3:
				GameOver();
				break;
			case 4:
				cls(1);
				return;
		}
	}
}



void PlayStartSound(void){
	int i;
	for(i=1;i<16;i++){
		PALETTEBUFF[i+256]=0x7FFF;
	}
	memset_(dmaoambuffer,0x2a0,0x100);
	memset_(MEM_VRAM+0x8000,0xFFFFFFFF,0x18);		//Ball and bats
	memset_(MEM_VRAM+0x8030,0,8);					//Ball and bats

	cpu_player = 0;
	cpuscore = 0;
	playerscore = 0;

	CountScore();
	state = 1;
}

void PlayPong(void){
	int playerpos = 80;
	int cpupos = 80;
	int ballxpos = 120;
	int ballypos = 80;
	int ballxvel = 0;
	int ballyvel = 0;
	int i,m;
	
	UpdateSprites(playerpos, cpupos, ballxpos, ballypos);
	sleepy(2);
	ballxvel = -2;

	while(state == 1){
		waitframe();
		AGBinput=~REG_P1;

//Player Control
		if(AGBinput & UP){
			playerpos -= 2;
			if(playerpos < 12) playerpos = 12;
		}
		if(AGBinput & DOWN){
			playerpos += 2;
			if(playerpos > 148) playerpos = 148;
		}
//Cpu Control
		if(ballypos < cpupos){
			cpupos -= 1;
			if(cpupos < 12) cpupos = 12;
		}
		if(ballypos > cpupos){
			cpupos += 1;
			if(cpupos > 148) cpupos = 148;
		}

		ballxpos += ballxvel;
		ballypos += ballyvel;
		if(ballypos <= 4){
			ballyvel = -ballyvel;
			ballypos += 4-ballypos;
		}
		if(ballypos >= (160-4)){
			ballyvel = -ballyvel;
			ballypos -= ballypos-(160-4);
		}
		if( (ballxpos < 20) && (ballxpos > 12) ){
			i = abs(ballypos-playerpos);
			m = ballypos-playerpos;
			if(i < 16){					// Did the ball hit the player bat
				ballxvel = -ballxvel;
				ballxpos += 20-ballxpos;
				DoRumble = 8;
				if( (i>>3) > 0 ){
					if(m > 0) ballyvel += 1;
					if(m < 0) ballyvel -= 1;
					if(ballyvel > 5) ballyvel = 5;
					if(ballyvel < -5) ballyvel = -5;
				}
			}
		}
		if( (ballxpos > 220) && (ballxpos < 228) ){
			i = abs(ballypos-cpupos);
			m = ballypos-cpupos;
			if(i < 16){					// Did the ball hit the cpu bat
				ballxvel = -ballxvel;
				ballxpos -= ballxpos-(220);
				if( (i>>3) > 0 ){
					if(m > 0) ballyvel += 1;
					if(m < 0) ballyvel -= 1;
					if(ballyvel > 5) ballyvel = 5;
					if(ballyvel < -5) ballyvel = -5;
				}
			}
		}
		if(ballxpos <= 4){
			state = 2;				//Count score
			cpu_player = 1;			//Cpu score
			DoRumble = 30;
		}
		if(ballxpos >= 236){
			state = 2;				//Count score
			cpu_player = 2;			//Player score
		}

		UpdateSprites(playerpos, cpupos, ballxpos, ballypos);
	}
}

void CountScore(void){
    char str[30];
    char *s=str+6;
    int mod;

	state = 1;					// More game

	if(cpu_player == 1){
		cpuscore++;
		if(cpuscore == 9){
			drawtext(8,"           You Lose           ",0);
			state = 3;			// GameOver
		}
	}
	if(cpu_player == 2){
		playerscore++;
		if(playerscore == 9){
			drawtext(8,"           You Win!           ",0);
			state = 3;			// GameOver
		}
	}
	cpu_player = 0;

	strcpy(str,"  P1:0               P2:0    ");
	mod=(playerscore&15);
	*(s++)=(mod+'0');
	s+=18;
	mod=(cpuscore&15);
	*(s++)=(mod+'0');

	drawtext(0,str,0);
	sleepy(2);
}

void GameOver(void){
	drawtext(10,"           GameOver           ",0);
	state = 4;					// Return to startscreen
	sleepy(2);
	memset_(dmaoambuffer,0x2a0,0x100);
}

void UpdateSprites(int ppos, int cpos, int bxpos, int bypos){
	
	dmaoambuffer[0] = (bypos-4) & 0xFF;
	dmaoambuffer[1] = (bxpos-4) & 0x1FF;
	dmaoambuffer[2] = 0;


	dmaoambuffer[4] = ((ppos-12) & 0xFF) | 0x8000;
	dmaoambuffer[5] = 8 | 0x4000;
	dmaoambuffer[6] = 0;

	dmaoambuffer[8] = ((cpos-12) & 0xFF) | 0x8000;
	dmaoambuffer[9] = 224 | 0x4000;
	dmaoambuffer[10] = 0;
}

int abs(int i){
	if(i<0) i = -i;
	return(i);
}
